from matplotlib.pyplot import title, imshow, ginput, imread, show, fill, draw, close, cla, clf, gca, axis, savefig, figure, ion, legend, text, xticks, yticks
import numpy as np
import sys,os
from time import sleep
CLASS_NAMES=['Shirt_collar','Shirt_sleeves','T-shirt_collar','T-shirt_sleeves','Jeans_hips','Jeans_pants hemline','Polo_collar','Polo_sleeves','Sweater_hood','Sweater_sleeves','Dress_collar']

def draw_figure(ima, ptslist):
    cla()
    imshow(ima)
    thelegend=[]
    axis("image")
    colors=['b','y','pink','orange','r','c','m','g','DarkOrchid','Ivory','GreenYellow']
    legend_types=CLASS_NAMES
    theflags={'o':'Occluded','b':'Bold','h':'Hard','d':'Disabled'}

    typespresent=[]
    ordertogo=[]
    dointheend=[]
    flagspresent=[]
    for i, pack in enumerate(ptslist):
        flag,typeG,pts=pack
        if not(typeG in typespresent):
            ordertogo.append(i)
            typespresent.append(typeG)
        else:
            dointheend.append(i)
    ordertogo.extend(dointheend)

    # draw all annotations in image
    for i in range(len(ptslist)):
        flag,typeG,pts = ptslist[ordertogo[i]]
        if len(pts.shape)<2:
            pts.shape=(pts.shape[0]/2,2)
        # draw annotation polyline
        fill(pts[:,0], pts[:,1], colors[int(typeG)-1], alpha=0.5, edgecolor='r')
        # add type to legend
        if not(legend_types[int(typeG)-1] in thelegend):
            thelegend.append(legend_types[int(typeG)-1])
        # draw visibility flag
        if not flag in ['n','N']:
            ax=gca()
            ax.annotate(flag, xy=(pts[0,0],pts[0,1]), bbox=dict(boxstyle="round", fc="0.8"))
            if flag not in flagspresent:
                flagspresent.append(flag)
    # draw legend
    legend(thelegend, shadow=True, fancybox=True, loc=0)
    # draw visibility legend
    if len(flagspresent)!=0:
        thetext=', '.join([x+': '+theflags[x] for x in flagspresent])
        text(10, 470,thetext, bbox=dict(boxstyle="round", fc="0.8"))
    ax=gca()
    xticks([])
    yticks([])
    draw()
#---------

def read_gtfile(gt_file, indict=False):
    pf  = open(gt_file,"r")
    lines = [lin for lin in pf.readlines() if lin[0]!='#']
    pf.close()

    # dict or list
    if indict:    gt_data = {}
    else:         gt_data = []

    for lin in lines:
        if lin.strip() == '':
            continue
        tokens = lin.split(' ')
        fname = tokens[0] #file name
        flag = tokens[1]  #visibility flag
        typeG = tokens[2] #type of garment
        pts=np.asarray([float(x) for x in tokens[3:]]) #polyline points
        if indict:
            if gt_data.has_key(fname):
                gt_data[fname].append((flag,typeG,pts))
            else:
                gt_data[fname]=[(flag,typeG,pts)]
        else:
            gt_data.append((fname,flag,typeG,pts))
    return gt_data
#-------------------

def about():
    print """ 
    Annotation visualization tool for the IRI Clothing Part Dataset.

    Usage:
    visualize_annotations.py [-s] [-p path_to_dataset] -f <gt file> 

    Description:
    -s
         flag to save images with annotations
    -p <path_to_dataset>
         optional path to the root directory of the dataset
    -f <gt file>
         file with ground truth annotations    
    """
    sys.exit()


if __name__=="__main__":
    save = False
    dataset_path = ''
    gt_file = None

    # Parse arguments
    args = sys.argv[1:]
    while len(args):
        a=args.pop(0)
        if a == '-s':
            save = True
        elif a == '-p':
            dataset_path = args.pop(0)
        elif a == '-f':
            gt_file = args.pop(0)
        else:
            print "ERROR: Invalid parameter", a
            about()
    if gt_file==None:
        print "ERROR: A ground truth file must be specified!"
        about()

    # load ground truth data
    if not os.path.exists(gt_file):
        if not os.path.exists(dataset_path+'/gt/'+gt_file):
            print "ERROR: Could not find", gt_file
            about()
        else:
            gt_file = dataset_path+'/gt/'+gt_file
    gt_data = read_gtfile(gt_file, True)

    # init figure
    figure(1)
    ion()
    
    # draw annotations
    keys=gt_data.keys()
    keys.sort()
    for fname in keys:
        ptslist  = gt_data[fname]
        ima_path = dataset_path+fname+'.png'
        ima      = imread(ima_path)
        draw_figure(ima, ptslist)

        if save:
            savefig(fname.split("/")[1], format="png",bbox_inches='tight')
        else:
            raw_input("FILE: %s.png -- Press ENTER to continue."%fname) #Not portable otherwise
